import { CustomRendererConfig } from './shared/renderer-hooks';
export interface Config {
    /**
     * Specification to use to determine which nodes in the template require custom renderer hooks.
     */
    customRendererConfig?: CustomRendererConfig;
    /**
     * Enable computed member expression in the template. eg:
     *    <template>
     *        {list[0].name}
     *    </template>
     */
    experimentalComputedMemberExpression?: boolean;
    /**
     * Enable <x-foo lwc:directive={expr}>
     */
    experimentalDynamicDirective?: boolean;
    /**
     * When true, HTML comments in the template will be preserved.
     */
    preserveHtmlComments?: boolean;
    /**
     * When false, the template compiler will not generate optimized code for static content.
     */
    enableStaticContentOptimization?: boolean;
    /**
     * When true, enables `lwc:spread` directive.
     */
    enableLwcSpread?: boolean;
    /**
     * When true, enables usage of `lwc:slot-bind` and `lwc:slot-data` directives for declaring scoped slots
     */
    enableScopedSlots?: boolean;
}
export type NormalizedConfig = Required<Omit<Config, 'customRendererConfig'>> & Partial<Pick<Config, 'customRendererConfig'>>;
export declare function normalizeConfig(config: Config): NormalizedConfig;
